// Jason Rohrer
// Hemicube.cpp

/**
*
*	Hemicube object for RadiosGL
*
*	Contains form factors for pixels in hemicube
*
*	Created 9-17-99
*	Mods:
*		Jason Rohrer	1-16-2000	Changed to use UniversalFileIO to work on both big 
*									and little endian machines
*
*/


#include "Hemicube.h"
#include "UniversalFileIO.h"

// constructor, fills hemicube arrays using file data
Hemicube::Hemicube(float worldW, FILE *hemicubeFile) {
	
	UniversalFileIO fileIO;
	
	worldWide = worldW;
	
	// read pixels wide of hemicube
	pixelsWide = fileIO.freadLong( hemicubeFile );
	
	// precalc offsets
	yOffsetPixels = new int[pixelsWide];
	
	for( int y=0; y<pixelsWide; y++) {
		int offset = y*pixelsWide;
		yOffsetPixels[y] = offset;
		}
		
	int regionSize = pixelsWide / 2;	
	
	int numPixelsTop = pixelsWide * pixelsWide;
	int numPixelsSide = numPixelsTop / 2;
	
	int numPixelsRegion = numPixelsSide / 2;
	
	// init arrays
	topPixels = new float[ numPixelsTop ];
	
	// note that form factors for corresponding pixels on each side are identical
	// side 0 only side that's actually an array
	sidePixels[0] = new float[ numPixelsSide ];	


	
	// read first region into top
	// must read one row at a time...
	for(int r=0; r<regionSize; r++) {
		// new way (using UniversalFileIO): must read one pixel at a time
		for( int q=0; q<regionSize; q++) {
			topPixels[yOffsetPixels[r] + q] = fileIO.freadFloat( hemicubeFile );
			}
		}
	// read second region into side 0
	// must read one row at a time...
	for(int r=0; r<regionSize; r++) {
		// new way (using UniversalFileIO): must read one pixel at a time
		for( int q=0; q<regionSize; q++) {
			sidePixels[0][yOffsetPixels[r] + q] = fileIO.freadFloat( hemicubeFile );
			}
		}
	// let caller close file
	
	
	// copy into rest of top, and into sides
	for(int fileY =0; fileY < regionSize; fileY++) {
		for( int fileX = 0; fileX < regionSize; fileX++) {		// mirror left to right
			topPixels[yOffsetPixels[fileY] + pixelsWide - fileX - 1] = topPixels[yOffsetPixels[fileY] + fileX];
			sidePixels[0][yOffsetPixels[fileY] + pixelsWide - fileX - 1] = sidePixels[0][yOffsetPixels[fileY] + fileX];		// side 0
			}
		for( int fileX = 0; fileX < pixelsWide; fileX++) {		// mirror top to bottom
			topPixels[yOffsetPixels[pixelsWide - fileY -1] + fileX] = topPixels[yOffsetPixels[fileY] + fileX];
			}
		}
	
	for(int i=0; i<4; i++) {	// put pointer to side 0 in other sides
		sidePixels[i] = sidePixels[0];
		}	
		
	}