// Jason Rohrer
// Output.h

/**
*
*	Object for outputing textures computed by RadiosGL
*
*
*	Created 11-14-99
*	Mods:
*		Jason Rohrer	12-5-99		Added adaptive patch texture sizing,(interface changed)
*		Jason Rohrer	12-16-99	When making a texture for a patch, all coplanar pathches that share
*										edges with the patch are also projected (interface changed)
*/




#ifndef OUTPUT_INCLUDED
#define OUTPUT_INCLUDED



#include <float.h>
#include <math.h>
#include <stdio.h>

#include "triangle.h"
#include "smoothTexture.h"

#include "Patch.h"
#include "PatchGroup.h"
#include "SimpleVector.h"


class Output {

	public:
		
		// takes:
		// file, array of patches, number of patches, textures max high, textures max wide, largest patch area
		Output( FILE *oFile, Patch *patchesIn, int nPatches, int maxTxHigh, int maxTxWide, float maxPArea, SimpleVector<int> *shrEdges );
		
		~Output();

		// build and output textures for patches based on these groups
		void buildFromGroups( SimpleVector<PatchGroup*> &groups );
		
	private:
		FILE *outFile;
	
		float maxPatchArea;
		
		long maxTextWide;
		long maxTextHigh;
		
		Patch *patches;
		
		long numPatches;		// number of patches in output file

		// arrays:
		// rotation around x axis to bring patch normal into yz plane
		float *thetaX;
		// rotation around y axis to bring patch normal onto z axis (patch aligned with xy plane)
		float *thetaY;
		// shift per patch to center it on texture
		float *shiftX;
		float *shiftY;
		// zoom per patch to make it reach all edges of texture
		float *zoomX;
		float *zoomY;
		// per patch anchor in texture of v0...v2
		long *textAnchorX[3];
		long *textAnchorY[3];	
		
		// texture sizes for each patch
		long *texturesHigh;
		long *texturesWide;
		
		
		// vectors for each patch that list coplanar patches that share edges
		SimpleVector<int> *shareEdges;
			
	};


inline Output::~Output() {
	
	delete [] texturesHigh;
	delete [] texturesWide;
	
	delete [] thetaX;
	delete [] thetaY;
	delete [] shiftX;
	delete [] shiftY;
	delete [] zoomX;
	delete [] zoomY;
	
	for( int i=0; i<3; i++ ) {
		delete [] textAnchorX[i];
		delete [] textAnchorY[i];
		}	
		
	}

#endif