// Jason Rohrer
// Radiosity.h

/**
*
*	Main radiosity object
*
*
*	Created 9-12-99
*	Mods:
*			Jason Rohrer	10-2-99		Added use of PatchGroups.	
*
*			Jason Rohrer	11-20-99	Added gamma correction to color normalization
*/


#ifndef RADIOSITY_INCLUDED
#define RADIOSITY_INCLUDED

#include "Patch.h"
#include "PatchGroup.h"
#include "Output.h"

#include <stdio.h>
#include <float.h>
#include <math.h>

class Radiosity {


	public:
		
		
		// files for model, hemicube, and output, size of hemicube in world coords, maximum patch area, threshold for ending iterative calculation, threshold for subdivision, gamma correction factor (default = no correction), texture size (power of 2)
		Radiosity(FILE *modlFile, FILE *hcFile, FILE *outFile, float hemicubeWorldWide, float maxPArea, float iterThresh, float subDivThresh, float gam=1.0, int textSize=64 );
		
		~Radiosity();
		
		void run();
		
	private:
		
		int numPatches;
		Patch *patches;
		
		SimpleVector<PatchGroup*> groups;			// subdivided patches
			
		Hemicube *hemiC;
		
		Output *outToFile;

		float iterThreshold;
		float subDivideThreshold;
		
		float gamma;
		
		float maxPatchArea;		// for initial patch subdivision before calculations
		
		SimpleVector<int> *shareEdges;		// array of vectors, one for each patch...  which patches does this patch share edges with?
	};


inline Radiosity::~Radiosity() {
	int numGroups = groups.size();
	for( int i=0; i<numGroups; i++) {			// delete memory locations that vector elements are pointing to
		delete (*groups.getElement(i));
		}
	
	delete [] shareEdges;
	delete [] patches;
	delete outToFile;
		
	// vector instance declared on stack, so no need to delete
	delete hemiC;
	}
	
#endif