// Jason Rohrer
// ViewCube.h

/**
*
*	ViewCube object for RadiosGL
*
*	patch specific hemicube "instance"  (points to global hemicube)
*
*
*	Created 9-12-99
*	Mods:
*		Jason Rohrer	12-1-99		Optimization:
*									Precalculate many values
*
*/



#ifndef VIEWCUBE_INCLUDED
#define VIEWCUBE_INCLUDED

#include "Vertex.h"
#include "Hemicube.h"
#include "ZPixel.h"
#include "Patch.h"

#include "zBufferPatch.h"
#include "RadiosityConstants.h"

#include <math.h>





class ViewCube {
	
	public:
		
		
		// center of cube, Normal of top, pointer to hemicube
		ViewCube(Vertex cVert, Vertex N, Hemicube *h);
		
		~ViewCube();
		
		void projPatchOnCube(int pID, Patch *p);	// add patch to viewcube and Z-buffer
		
		
		// fill array formFactors[] with summed form factors from pixel form factors in cube
		// assumes formFactors[] inited to size of patch ID range
		void sumFormFactors(float *formFactors);
		
		void print();	// print out top of view cube
		
	private :
	
		ZPixel *topPixels;		// array of patch IDs at each pixel on cube
		ZPixel *sidePixels[4];
		
		Vertex centVert;		// position of center of cube in world space
		Vertex norm;
		
		int numPixelsWide;
		
		float worldToPixUnit;	// conversion factor from world coords to pixels
		
		Hemicube *hCube;		// pointer to global hemicube
		
		
		int *yOffset;
		
		
		
		// precalculated as an optimization
		
		
		// based on width of hemicube
		float halfWorldWide;
		float startXWorld;
		float endXWorld;
		float startYWorld;
		float endYWorld;
		float startSideYWorld;	// sides of view cube are "shorter"
		
		// difference between cube center and origin
		Vertex deltaCenter;
		
		// rotation needed to line normal up with Z+
		float thetaX;
		float thetaY;
		
		
		
		
	};
	
	
#endif