// Jason Rohrer
// smoothTexture.cpp

/**
*
*	image smoothing function:  Naive (doesn't weight average)
*		Does some special case things with the corners and edges.... should be reviewed.
*	Originally done for spring 99 graphix framework
*
*	Leaves alpha channel alone...  actually, seems to drop alpha... should fix this...
*
*	Created ( spring 99 ?)
*	Mods:
*		Jason Rohrer	12-5-99		Fixed to use more standard C++ conventions (like new and delete)
*									Complete overhaul of algorithm.  Code rewritten to be shorter and cleaner
*/





#include "smoothTexture.h"

// smooth a 32 bit texture
void smoothTexture( short textH, short textW, unsigned long* texturePtr, char smoothRadius ) { 					

	//	textH =  texture height
	//	textW = texture width
	//	texturePtr = pointer to texture in memory
	//	smoothRadius =  radius in pixels of the smoothing
	
	
	unsigned long* tempPtr;
	
	//unsigned long color;
	short red;
	short green;
	short blue;
	short x;
	short y;
	short maxY;
	short maxX;
	short minY;
	short minX;
	long pixelsInSum;
	
	long *yIndex;

	
	tempPtr = new unsigned long[textH * textW];
	
	yIndex = new long[ textH ];
	
	// copy texture into temp buffer
	for( y=0; y<textH; y++) {
		yIndex[y] = y*textW;		// precalc y index
		for( x=0; x<textW; x++) {
			tempPtr[yIndex[y] + x] = texturePtr[yIndex[y] + x];
			}
		}
	
	
	
	
	for( y=0; y<textH; y++ ) {		// for each pixel in texture
		int startY = y - smoothRadius;
		int endY = y + smoothRadius;
		
		if( startY < 0 )  startY = 0;		// watch for radius violating boundaries
		if( endY >= textH ) endY = textH - 1;
		
		int yContrib = yIndex[y];
		
		for( x=0; x<textW; x++ ) {
			
			long alphaSum = 0;
			long redSum = 0;
			long greenSum = 0;
			long blueSum = 0;
			
			
			int startX = x - smoothRadius;
			int endX = x + smoothRadius;
	
			if( startX < 0 )  startX = 0;		// watch for radius violating boundaries
			if( endX >= textW ) endX = textW - 1;
	
			int numPixelsInAverage = (endY - startY + 1) * (endX - startX + 1);
			
			for( int ry=startY; ry<=endY; ry++ ) {		// for each pixel in radius
				int rYContrib = yIndex[ry];
				for( int rx=startX; rx<=endX; rx++ ) {
					unsigned long color = tempPtr[rYContrib + rx];
					
					int alpha = (color >> 24) & 0x000000FF;		// extract components
					int red = (color >> 16) & 0x000000FF;
					int green = (color >> 8) & 0x000000FF;
					int blue = color & 0x000000FF;
					
					alphaSum += alpha;			// sum components
					redSum += red;
					greenSum += green;
					blueSum += blue;
					
					}
				}
		
			float invPixelsInAverage = 1.0 / (float) numPixelsInAverage;
			
			alphaSum = (int)(alphaSum * invPixelsInAverage);		// divide to complete average
			redSum = (int)(redSum * invPixelsInAverage);
			greenSum = (int)(greenSum * invPixelsInAverage);
			blueSum = (int)(blueSum * invPixelsInAverage);
	
			// write average into texture
			texturePtr[ yContrib + x] = blueSum | (greenSum<<8) | (redSum<<16) |  (alphaSum<<24);
			}
		}
	}	
		
	