// Jason Rohrer
// vrml2mdl.cpp

/**
*
*	Converts VRML file to a .MDL file for radiosity.
*	Works only for files containing triangles with diffuse color and PER_FACE material binding
*
*	Created 12-5-99
*	Mods:
*
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "Vertex.h"
#include "Color.h"
#include "SimpleVector.h"


// prototypes
int mainFake( int argc, char *argv[] );

void lookForStringInFile( FILE *f, unsigned char *s, unsigned char *stringBuff );
void addVRMLDataToMDL( FILE *vrmlFILE, FILE *mdlFILE, char isLighting);

void usage();
void unknownOption(char *s);
void unknownArgument(char *s);


char *paramsFileName = "vrml2mdl.ini";

int main (int argc, char *argv[] ) {


	printf("vrml2mdl\n\n");
	
	FILE *vrmlLightFILE;		// file with light-emitting patches
	FILE *vrmlNonLightFILE;		// file with reflectance-only patches
	FILE *mdlFILE;

	int vrmlLightFileArgNumber = 0;
	int vrmlNonLightFileArgNumber = 0;
	int mdlFileArgNumber = 0;

	int numPatches = 0;


	int clearArgV = false;		// argv needs deallocation?
	
	if( argc == 0 ) {	// read params from a file
		printf( "No input arguments specified, trying read them from file:  %s\n\n", paramsFileName );
		// try opening the file
		FILE *paramsFile = fopen( paramsFileName, "r" );	// a text file
		
		if( paramsFile == NULL ) {	// file open failed
			printf("Could not open file for reading:  %s\n\n", paramsFileName);
			usage();
			}
		
		
		// read params from file, stick them into arg array
		
		delete [] argv;
		
		argv = new char *[99];		// make it big enough to hold args
		
		int retVal = 1;
		int v = 0;
		
		// note:  skip first argument slot, since this is where name of executable is kept
		
		while( retVal != 0 && retVal != EOF ) {
			v++;
			argv[v] = new char[99];
			retVal = fscanf( paramsFile, "%s", argv[v] );
			}
		// last v-index in argv is not valid arg, so v = number of args in argv
		argc = v;
		
		clearArgV = true;		// set flag to deallocate argv memory
		}




	if( argc < 7 ) {		// 3 input values (4 params, counting switches)
		usage();
		}

	// argc >= 7
	int i=1;
	
	while( i<argc ) {
		switch( argv[i][0] ) {
			case '-':			// read any switches
				switch( argv[i][1] ) {
					case '?':
						usage();
						break;
					case 'l':
						i++;
						vrmlLightFileArgNumber = i;
						i++;				
						break;	
					case 'r':
						i++;
						vrmlNonLightFileArgNumber = i;
						i++;				
						break;
					case 'm':
						i++;
						mdlFileArgNumber = i;
						i++;				
						break;
					case 'n':
						i++;
						sscanf(argv[i], "%d", &numPatches);
						i++;				
						break;
					default:		// handle bad switches
						unknownOption(argv[i]);
						break;
					}
				break;
			default:			// handle bad arguments
				unknownArgument(argv[i]);
				i++;
				break;
			}
		}
	
	
	if( numPatches <= 0 ) {
		printf("total_num_patches must be greater than 0\n\n");
		usage();
		}
	
	// params read correctly
	
	
	// clear argv if need be
	if( clearArgV ) {
		for( int p=0; p<argc; p++ ) {
			delete [] argv[p];
			}
		delete [] argv;
		}
	
	
	// open files
	vrmlLightFILE = fopen((const char*)argv[vrmlLightFileArgNumber], "r");
	if( vrmlLightFILE == NULL ) {
		printf("Could not open file for reading:  %s\n\n", (const char*)argv[vrmlLightFileArgNumber]);
		usage();
		}
	vrmlNonLightFILE = fopen((const char*)argv[vrmlNonLightFileArgNumber], "r");
	if( vrmlNonLightFILE == NULL ) {
		printf("Could not open file for reading:  %s\n\n", (const char*)argv[vrmlNonLightFileArgNumber]);
		usage();
		}
	mdlFILE = fopen((const char*)argv[mdlFileArgNumber], "w");
	if( mdlFILE == NULL ) {
		printf("Could not open file for writing:  %s\n\n", (const char*)argv[mdlFileArgNumber]);
		usage();
		}
	
	// all files open
	
	printf( "Merging %d patches from %s and %s into %s\n...\n", numPatches, 
						argv[vrmlLightFileArgNumber], argv[vrmlNonLightFileArgNumber],
						argv[mdlFileArgNumber] );
	
	// write out number of patches
	fprintf( mdlFILE, "%d\n\n", numPatches );
	
	addVRMLDataToMDL( vrmlLightFILE, mdlFILE, true );
	addVRMLDataToMDL( vrmlNonLightFILE, mdlFILE, false );
	
	printf("\n\ndone");
	return 0;
	}	
	


// convert data from VRML file and append to end of mdl file
// isLighting is 	true if colors in VRML should be emittances in MDL
//				 	false if colors in VRML should be reflectances in MDL
void addVRMLDataToMDL( FILE *vrmlFILE, FILE *mdlFILE, char isLighting) {
	// read points from vrml file
	SimpleVector<Vertex*> pointVector;
	
	unsigned char *inString = new unsigned char[100];
	
	// look for Coordinate3
	lookForStringInFile( vrmlFILE, (unsigned char *)"Coordinate3", inString );
	
	// Coordinate3 found, assume there is at least one point listed
	
	fscanf(vrmlFILE, " { point [ ");
	
	char morePoints = true;
	
	float x, y, z;
	
	char *nextChar = new char[100];
	
	while( morePoints ) {
		
		fscanf( vrmlFILE, "%f %f %f", &x, &y, &z);
	
		// add point to vector
		pointVector.push_back( new Vertex( x, y, z ) );
		
		// check for more points
		fscanf( vrmlFILE, "%99s", nextChar );
		
		if( nextChar[0] == ',' ) {
			morePoints = true;		// points are comma delimted
			}
		else {
			morePoints = false;		// no more points
			}
		}
	
	// all points read from file	
	
	
	// read points from vrml file
	SimpleVector<Color*> colorVector;
		
	// now look for Material
	lookForStringInFile( vrmlFILE, (unsigned char*)"Material", inString );
	
	// Material found, assume there is at least one color listed
	
	fscanf(vrmlFILE, " { diffuseColor [ ");
	
	char moreColors = true;
	float r, g, b;
	
	int numColors = 0;
	
	while( moreColors ) {
		
		fscanf( vrmlFILE, "%f %f %f", &r, &g, &b);
		
		numColors++;
	
		// add point to vector
		colorVector.push_back( new Color( r, g, b ) );
		
		// check for more points
		fscanf( vrmlFILE, "%99s", nextChar );
		
		if( nextChar[0] == ',' ) {
			moreColors = true;		// colors are comma delimted
			}
		else {
			moreColors = false;		// no more colors
			}
		}
	
	// all colors read from file

	
	// now, finally, read patches from file and write out simultaneously
	
	// look for IndexedFaceSet
	lookForStringInFile( vrmlFILE, (unsigned char *)"IndexedFaceSet", inString );
	
	// IndexedFacetSet found, assume there is at least one patch
	
	fscanf(vrmlFILE, " { coordIndex [ ");
	
	char morePatches = true;
	int v[3], d;		// indices into list of vertices
	
	int numPatches = 0;
	
	while( morePatches ) {
		fscanf( vrmlFILE, "%d, %d, %d, %d", &(v[0]), &(v[1]), &(v[2]), &(d));
		
		fprintf( mdlFILE, "\n\n" );
		for( int i=0; i<3; i++ ) {	// write each vertex to file
			
			Vertex *thisVert = *(pointVector.getElement( v[i] ));
			
			// print vertex into mdl file
			fprintf( mdlFILE, "%f %f %f\n", thisVert->x, thisVert->y, thisVert->z );
			}
		
		fprintf( mdlFILE, "\n" );
		
		
		
		// now print color to file
		
		Color *thisColor = *(colorVector.getElement( numPatches ));
		
		// depending on isLighting, put 0,0,0 color before or after color data
		
		if( isLighting ) {	// reflectances 0 if lighting mode
			fprintf( mdlFILE, "%f %f %f\n", 0, 0, 0 );
			}
		
		fprintf( mdlFILE, "%f %f %f\n", thisColor->r, thisColor->g, thisColor->b );
		
		if( !isLighting ) {	// emittances 0 if not lighting mode
			fprintf( mdlFILE, "%f %f %f\n", 0, 0, 0 );
			}
		
		numPatches++;
		
		// check for more points
		fscanf( vrmlFILE, "%99s", nextChar );
		
		if( nextChar[0] == ',' ) {
			morePatches = true;		// patches are comma delimted
			}
		else {
			morePatches = false;		// no more patches
			}
		
		}
	
	// done..

	// free memory
	
	delete [] inString;
	delete [] nextChar;
	
	// delete elements from point vector
	int numPoints = pointVector.size();
	for( int p=0; p<numPoints; p++ ) {
		Vertex *thisVert = *(pointVector.getElement( p ));
		
		delete thisVert;
		}
	
	// delete elements from color vector
	numColors = colorVector.size();
	for( int c=0; c<numPoints; c++ ) {
		Color *thisColor = *(colorVector.getElement( c ));
		
		delete thisColor;
		}

	}





	
// uses string buff of size 100 to read from file and compare to s
// exits w/ message if string not found
void lookForStringInFile( FILE *f, unsigned char *s, unsigned char *stringBuff ) {
	char found = false;
	while( !found ) {
		fscanf( f, "%99s", stringBuff );
		if( 0 == strcmp ((const char *)stringBuff, (const char *)s) ) {
			found = true;
			}
		
		if( stringBuff[0] == EOF ) {		// end of file
			printf("VRML missing required data (no %s block)\n", s);
			exit(0);
			}
		}
	}
	

	
	

void usage() {				// print usage help message and exit

	printf("usage:\n");
	printf("vrml2mdl\n");
	printf("\t[-?]\n\t");
	printf("-l vrml_lighting_patch_file\n\t");
	printf("-r vrml_reflectance_patch_file\n\t");
	printf("-m mdl_file -n total_num_patches\n\t");
	printf("example:\n\t");
	printf("vrml2mdl -l testLights.vrml -r testPatches.vrml -m out.mdl -n 24\n\n\t");
	
	exit(0);
	}
	
void unknownOption(char *s) {
	printf("unknow option:  %s\n\n", s);
	usage();
	}
	
void unknownArgument(char *s) {
	printf("unknow argument:  %s\n\n", s);
	usage();
	}


